from rest_framework import serializers
from rest_framework_simplejwt.tokens import RefreshToken


class LogoutSerializer(serializers.Serializer):
    refresh = serializers.CharField()

    def validate(self, attrs):
        self.token = attrs["refresh"]
        return attrs

    def save(self, **kwargs):
        try:
            token = RefreshToken(self.token)
            token.blacklist()
        except Exception:
            raise serializers.ValidationError("Invalid or expired token.")



from rest_framework.response import Response
from rest_framework import status, permissions
from ..serializers.logout import LogoutSerializer
from rest_framework.generics import GenericAPIView

class LogoutView(GenericAPIView):
    serializer_class = LogoutSerializer
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(
            {"detail": "Successfully logged out."},
            status=status.HTTP_200_OK,
        )


from typing import TYPE_CHECKING

from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import UserManager as DjangoUserManager

if TYPE_CHECKING:
    from src.bunifu_django_auth.models.users import AbstractBunifuUser  # noqa: F401



class UserManager(DjangoUserManager["AbstractBunifuUser"]):
    """Custom manager for the User model."""
    use_in_migrations = True

    def _create_user(self, email: str, password: str | None, **extra_fields):
        """
        Create and save a user with the given email and password.
        """
        if not email:
            msg = "The given email must be set"
            raise ValueError(msg)
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.password = make_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email: str, password: str | None = None, **extra_fields):  # type: ignore[override]
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_superuser", False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email: str, password: str | None = None, **extra_fields):  # type: ignore[override]
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)

        if extra_fields.get("is_staff") is not True:
            msg = "Superuser must have is_staff=True."
            raise ValueError(msg)
        if extra_fields.get("is_superuser") is not True:
            msg = "Superuser must have is_superuser=True."
            raise ValueError(msg)

        return self._create_user(email, password, **extra_fields)




from rest_framework.generics import CreateAPIView
from ..serializers.register import RegisterSerializer
from ...models.users import BunifuUser


class RegisterView(CreateAPIView):
    serializer_class = RegisterSerializer
    queryset = BunifuUser.objects.all()



import pytest
from rest_framework.test import APIClient
from ..factories.users import UserFactory


@pytest.mark.django_db
def test_user_account_login():
    raw_password = "password123"
    user = UserFactory(password=raw_password)
    client = APIClient()
    
    response = client.post(
        "/login/",
        {"email": user.email, "password": raw_password},
        format="json",
    )
    assert response.status_code == 200